@@ -4,6 +4,7 @@ import logging
 from aiogram import Bot
 from aiogram.types.base import TelegramObject
 from aiogram.dispatcher import FSMContext
+from aiogram.dispatcher.handler import _check_spec, _get_spec
 
 from ..filters.filters import tg_obj_matches_filters
 from ..utils import class_is_original
@@ -15,14 +16,29 @@ class MetaBaseView(type):
         if class_is_original(cls, 'aiogram_oop_framework'):
             return
         methods_with_filters = []
-        for _, method in cls.__dict__.items():
-            if not isinstance(method, (classmethod, staticmethod)):
+        pre_middlwares = []
+        post_middlwares = []
+
+        def append_methods_with_filters(cls_):
+            for _, method in cls_.__dict__.items():
+                if not isinstance(method, (classmethod, staticmethod)):
+                    continue
+                func = method.__func__
+                if hasattr(func, "__execute_filters__"):
+                    methods_with_filters.append(method)
+                elif func.__name__.startswith("pre_"):
+                    pre_middlwares.append(method)
+                elif func.__name__.startswith("post_"):
+                    post_middlwares.append(method)
+
+        for base in cls.__bases__:
+            if class_is_original(base, 'aiogram_oop_framework'):
                 continue
-            func = method.__func__
-            if not hasattr(func, "__execute_filters__"):
-                continue
-            methods_with_filters.append(method)
+            append_methods_with_filters(base)
+        append_methods_with_filters(cls)
         cls._methods_with_filters = methods_with_filters
+        cls._pre_middlwares = pre_middlwares
+        cls._post_middlwares = post_middlwares
 
 
 class BaseView(metaclass=MetaBaseView):
@@ -71,25 +87,47 @@ class BaseView(metaclass=MetaBaseView):
     @classmethod
     async def _execute(cls, tg_obj: TelegramObject, state: FSMContext = None, **kwargs):
         """
-        this is a private method used by the framework for making filters like execute_in_private work
+        this is a private method used by the framework for making filters and middlwares work
 
         :param tg_obj: Message, CallbackQuery, InlineQuery etc
         :param state: FSMContext
         :param kwargs: kwargs
         :return:
         """
+
+        executed = False
+
+        for method in cls._pre_middlwares:
+            func = method.__func__
+            partial_data = cls.get_partial_data(func, **kwargs)
+            await func(cls, tg_obj, **partial_data)
+
         for method in cls._methods_with_filters:
             func = method.__func__
             filters = func.__execute_filters__
             if not filters:
                 continue
             if await tg_obj_matches_filters(tg_obj, filters):
+                partial_data = cls.get_partial_data(func, **kwargs)
                 if isinstance(method, classmethod):
-                    await func(cls, tg_obj)
+                    await func(cls, tg_obj, **partial_data)
                 else:
                     logging.warning("using staticmethods is deprecated in version 0.2.dev2, "
                                     "use only classmethods")
-                    await func(tg_obj)
-                return
+                    await func(tg_obj, **partial_data)
+                executed = True
+                break
 
-        await cls.execute(tg_obj, state, **kwargs)
+        if not executed:
+            partial_data = cls.get_partial_data(cls.execute, state=state, **kwargs)
+            await cls.execute(tg_obj, **partial_data)
+
+        for method in cls._post_middlwares:
+            func = method.__func__
+            partial_data = cls.get_partial_data(func, **kwargs)
+            await func(cls, tg_obj, **partial_data)
+
+    @classmethod
+    def get_partial_data(cls, func, **data):
+        spec = _get_spec(func)
+        return _check_spec(spec, data)
